<!DOCTYPE html lang="en">
<html lang="en">

<head>
        <title>PKLX</title>
        <script type="text/javascript" src="https://cdnjs.cloudflare.com/ajax/libs/vis/4.21.0/vis.js"></script>
        <link type="text/css" href="https://cdnjs.cloudflare.com/ajax/libs/vis/4.21.0/vis.css" rel="stylesheet" />
        <script type="text/javascript" src="static/visualisation.js"></script>
        <link type="text/css" href="static/index.css" rel="stylesheet" />
        <link rel="shortcut icon" href="static/favicon.ico" />
        <script src="https://ajax.googleapis.com/ajax/libs/jquery/3.3.1/jquery.min.js"></script>
        <link href="https://cdn.jsdelivr.net/npm/select2@4.0.13/dist/css/select2.min.css" rel="stylesheet" />
        <script src="https://cdn.jsdelivr.net/npm/select2@4.0.13/dist/js/select2.min.js"></script>
        <link rel="apple-touch-icon" sizes="180x180" href="{{ url_for('static', filename='apple-touch-icon.png') }}">
        <link rel="icon" type="image/png" sizes="32x32" href="{{ url_for('static', filename='favicon-32x32.png') }}">
        <link rel="icon" type="image/png" sizes="16x16" href="{{ url_for('static', filename='favicon-16x16.png') }}">
        <link rel="manifest" href="/site.webmanifest">
</head>

<body onload="start();">
        <div class="fixed-header">
                <div id="frame"> <select id="names_box" class="selectpicker" data-live-search="true"></select> </div>
                <a id="downloadAnchorElem" style="display:none"></a>
                <div id="statement">Loading, please wait...</div>
        </div>

        <div id="network">
                <div class="loader"></div>
        </div>

        <div class="fixed-footer">
                <div align="center" id="statement2">Graph visualisation by <a href="http://visjs.org">vis.js</a>. Created by <a href="http://phylomatx.com">PhylomatX</a></div>
        </div>
</body>
</html>
