from setuptools import setup

setup(
    name='PKLX',
    version='0.0.3',
    author='PhylomatX',
    author_email='phylomatx@gmail.com',
    description='PKLX',
    install_requires=['networkx==3.1', 'flask==2.3.2'],
)